@@ -1,4 +1,4 @@
-require 'rexml_test_utils'
+require_relative 'rexml_test_utils'
 require 'rexml/document'
 
 class TestIssuezillaParsing < Test::Unit::TestCase
